<?php
/**
 * @package   DPBexio
 * @copyright Copyright (C) 2022 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */

namespace DigitalPeak\Plugin\User\DPBexio\Extension;

use DigitalPeak\Library\DPBexio\Bexio\API;
use Exception;
use Joomla\CMS\Application\CMSApplicationInterface;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\Event\DispatcherInterface;
use Joomla\Event\Event;
use Joomla\Event\SubscriberInterface;
use stdClass;

final class DPBexio extends CMSPlugin implements SubscriberInterface
{
	public static function getSubscribedEvents(): array
	{
		return ['onUserAfterSave' => 'afterSave', 'onUserBeforeSave' => 'beforeSave'];
	}

	protected $autoloadLanguage = true;

	/** @var CMSApplicationInterface $app */
	protected $app;
	private $api;
	private $user;

	public function __construct(API $api, DispatcherInterface $subject, $config = [])
	{
		parent::__construct($subject, $config);

		$this->api = $api;
	}

	public function beforeSave(Event $event)
	{
		$this->user = (object)$event->getArgument(0);
	}

	public function afterSave(Event $event)
	{
		$user = (object)$event->getArgument(0);
		if (!$user->id) {
			return;
		}

		$users = $this->api->callBexioAPI('/3.0/users', [], $this->params, 'get')->data;
		if (empty($users)) {
			throw new Exception('Could not fetch users to create data for.', 404);
		}

		// Compile the data
		$contactData = [
			'contact_type_id' => 2,
			'name_1'          => $user->name,
			'mail'            => $user->email,
			'user_id'         => $users[0]->id,
			'owner_id'        => $users[0]->id
		];

		$this->addField('name_2', 'field_prename', $user, $contactData);
		$this->addField('address', 'field_street', $user, $contactData);
		$this->addField('postcode', 'field_postcode', $user, $contactData);
		$this->addField('city', 'field_city', $user, $contactData);

		// Compile the salutations for fast access
		$salutations = [];
		foreach ($this->api->callBexioAPI('/2.0/salutation', [], $this->params, 'get')->data as $salutation) {
			$salutations[$salutation->name] = $salutation->id;
		}
		$this->addField('salutation_id', 'field_salutation', $user, $contactData, $salutations);

		// Compile the countries for fast access
		$countries = [];
		foreach ($this->api->callBexioAPI('/2.0/country', [], $this->params, 'get')->data as $country) {
			$countries[$country->name] = $country->id;
		}
		$this->addField('country_id', 'field_country', $user, $contactData, $countries);

		// Search for the contact
		$contacts = $this->api->callBexioAPI(
			'/2.0/contact/search',
			[['field' => 'mail', 'value' => $this->user && !empty($this->user->email) ? $this->user->email : $user->email, 'criteria' => '=']],
			$this->params,
			'post'
		)->data;

		// Create or update the contact
		$this->api->callBexioAPI('/2.0/contact' . (!empty($contacts) ? '/' . $contacts[0]->id : ''), $contactData, $this->params, 'post');
	}

	/**
	 * Adds the given field to the contact data for he given parameter.
	 *
	 * @param string   $bexioName
	 * @param string   $param
	 * @param stdClass $user
	 * @param array    $contactData
	 * @param array    $replacement
	 */
	private function addField(string $bexioName, string $param, stdClass $user, array &$contactData, $replacement = []): void
	{
		// Check if there are fields
		if (empty($user->com_fields)) {
			return;
		}

		// Check if the param has a value
		if (!$name = $this->params->get($param)) {
			return;
		}

		// Check if the custom fields do contain the value
		if (!array_key_exists($name, $user->com_fields)) {
			return;
		}

		// When there is a fixed set of possibilities but it doesn't exist, don't update
		if ($replacement && !array_key_exists($user->com_fields[$name], $replacement)) {
			return;
		}

		// Set the data
		$contactData[$bexioName] = $user->com_fields[$name];

		// When there is a replacment, set it
		if (array_key_exists($user->com_fields[$name], $replacement)) {
			$contactData[$bexioName] = $replacement[$user->com_fields[$name]];
		}
	}
}
